@@ -28,7 +28,6 @@ from strawberry.exceptions import (
     UnallowedReturnTypeForUnion,
     WrongReturnTypeForUnion,
 )
-from strawberry.scalars import SCALAR_TYPES
 from strawberry.type import StrawberryType
 
 
@@ -44,7 +43,7 @@ class StrawberryUnion(StrawberryType):
         type_annotations: Tuple["StrawberryAnnotation", ...] = tuple(),
         description: Optional[str] = None,
     ):
-        self._name = name
+        self.graphql_name = name
         self.type_annotations = type_annotations
         self.description = description
 
@@ -52,7 +51,7 @@ class StrawberryUnion(StrawberryType):
         if isinstance(other, StrawberryType):
             if isinstance(other, StrawberryUnion):
                 return (
-                    self.name == other.name
+                    self.graphql_name == other.graphql_name
                     and self.type_annotations == other.type_annotations
                     and self.description == other.description
                 )
@@ -64,20 +63,6 @@ class StrawberryUnion(StrawberryType):
         # TODO: Is this a bad idea? __eq__ objects are supposed to have the same hash
         return id(self)
 
-    @property
-    def name(self) -> str:
-        if self._name is not None:
-            return self._name
-
-        name = ""
-        for type_ in self.types:
-            if hasattr(type_, "_type_definition"):
-                name += type_._type_definition.name  # type: ignore
-            else:
-                name += type.__name__
-
-        return name
-
     @property
     def types(self) -> Tuple[StrawberryType, ...]:
         return tuple(
@@ -150,9 +135,16 @@ class StrawberryUnion(StrawberryType):
 
             from strawberry.types.types import TypeDefinition
 
-            # Make sure that the type that's passed in is an Object type
+            # If the type given is not an Object type, try resolving using `is_type_of`
+            # defined on the union's inner types
             if not hasattr(root, "_type_definition"):
-                # TODO: If root=python dict, this won't work
+                for inner_type in type_.types:
+                    if inner_type.is_type_of is not None and inner_type.is_type_of(
+                        root, info
+                    ):
+                        return inner_type.name
+
+                # Couldn't resolve using `is_type_of``
                 raise WrongReturnTypeForUnion(info.field_name, str(type(root)))
 
             return_type: Optional[GraphQLType]
@@ -186,9 +178,17 @@ class StrawberryUnion(StrawberryType):
         return _resolve_union_type
 
 
+Types = TypeVar("Types", bound=Type)
+
+
+# We return a Union type here in order to allow to use the union type as type
+# annotation.
+# For the `types` argument we'd ideally use a TypeVarTuple, but that's not
+# yet supported in any python implementation (or in typing_extensions).
+# See https://www.python.org/dev/peps/pep-0646/ for more information
 def union(
-    name: str, types: Tuple[Type, ...], *, description: str = None
-) -> StrawberryUnion:
+    name: str, types: Tuple[Types, ...], *, description: str = None
+) -> Union[Types]:  # type: ignore
     """Creates a new named Union type.
 
     Example usages:
@@ -205,14 +205,9 @@ def union(
         raise TypeError("No types passed to `union`")
 
     for _type in types:
-        if _type in SCALAR_TYPES:
-            raise InvalidUnionType(
-                f"Scalar type `{_type.__name__}` cannot be used in a GraphQL Union"
-            )
-
         if not isinstance(_type, TypeVar) and not hasattr(_type, "_type_definition"):
             raise InvalidUnionType(
-                f"Union type `{_type.__name__}` is not a Strawberry type"
+                f"Type `{_type.__name__}` cannot be used in a GraphQL Union"
             )
 
     union_definition = StrawberryUnion(
@@ -221,4 +216,4 @@ def union(
         description=description,
     )
 
-    return union_definition
+    return union_definition  # type: ignore
